// LTTng 2.13 prepends the lttng_ust_ and LTTNG_UST_ prefix to all names
// to offer a consistent API namespace. Prefer the pre 2.13 behavior (API v0).
#define LTTNG_UST_COMPAT_API_VERSION 0

#undef TRACEPOINT_PROVIDER
#define TRACEPOINT_PROVIDER diary

#undef TRACEPOINT_INCLUDE
#define TRACEPOINT_INCLUDE "diary-tp.h"

#if !defined(_DIARY_TP_H) || defined(TRACEPOINT_HEADER_MULTI_READ)
#define _DIARY_TP_H

#include <time.h>
#include <lttng/tracepoint.h>

// Instrument a C user application
// https://lttng.org/docs/v2.13/#doc-c-application

TRACEPOINT_EVENT(
    diary,
    error,
    TP_ARGS(
        char*, msg_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    error_string,
    TP_ARGS(
        char*, msg_arg,
        const char*, arg1
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_string(arg1, arg1)
    )
)

TRACEPOINT_EVENT(
    diary,
    error_date,
    TP_ARGS(
        char*, msg_arg,
        const struct tm*, date_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_string(date_arg, asctime(date_arg))
    )
)

TRACEPOINT_EVENT(
    diary,
    error_int_long,
    TP_ARGS(
        char*, msg_arg,
        int, n_arg,
        long, m_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_integer(int, n, n_arg)
        ctf_integer(long, m, m_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug,
    TP_ARGS(
        char*, msg_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug_string,
    TP_ARGS(
        char*, msg_arg,
        char*, arg1
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_string(arg1, arg1)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug_int,
    TP_ARGS(
        char*, msg_arg,
        int, n_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_integer(int, n, n_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug_double,
    TP_ARGS(
        char*, msg_arg,
        double, n_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_integer(double, n, n_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug_sizet,
    TP_ARGS(
        char*, msg_arg,
        size_t, n_arg
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_integer(size_t, n, n_arg)
    )
)

TRACEPOINT_EVENT(
    diary,
    debug_tm,
    TP_ARGS(
        char*, msg_arg,
        const struct tm*, date
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_integer(int, tm_sec,   date->tm_sec)
        ctf_integer(int, tm_min,   date->tm_min)
        ctf_integer(int, tm_hour,  date->tm_hour)
        ctf_integer(int, tm_mday,  date->tm_mday)
        ctf_integer(int, tm_mon,   date->tm_mon)
        ctf_integer(int, tm_year,  date->tm_year)
        ctf_integer(int, tm_wday,  date->tm_wday)
        ctf_integer(int, tm_yday,  date->tm_yday)
        ctf_integer(int, tm_isdst, date->tm_isdst)
    )
)

TRACEPOINT_EVENT(
    diary,
    warning_string,
    TP_ARGS(
        char*, msg_arg,
        char*, arg1
    ),
    TP_FIELDS(
        ctf_string(msg, msg_arg)
        ctf_string(arg1, arg1)
    )
)

/* Log level assignment                     */
// TRACE_ERR
// TRACE_WARNING
// TRACE_INFO
// TRACE_DEBUG_LINE (default)
TRACEPOINT_LOGLEVEL(diary, error, TRACE_ERR)
TRACEPOINT_LOGLEVEL(diary, error_string, TRACE_ERR)
TRACEPOINT_LOGLEVEL(diary, error_date, TRACE_ERR)
TRACEPOINT_LOGLEVEL(diary, error_int_long, TRACE_ERR)
TRACEPOINT_LOGLEVEL(diary, debug, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, debug_string, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, debug_int, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, debug_double, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, debug_sizet, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, debug_tm, TRACE_DEBUG)
TRACEPOINT_LOGLEVEL(diary, warning_string, TRACE_WARNING)

#endif /* _DIARY_TP_H */

#include <lttng/tracepoint-event.h>
